@@ -1,8 +1,8 @@
 ------------------------------------------------------------------------------
 --
---  LGI Lua-side core module selector
+--  lgi Lua-side core module selector
 --
---  Copyright (c) 2010, 2011 Pavel Holejsovsky
+--  Copyright (c) 2010, 2011, 2015 Pavel Holejsovsky
 --  Licensed under the MIT license:
 --  http://www.opensource.org/licenses/mit-license.php
 --
@@ -11,4 +11,12 @@
 -- This module decides what kind of core routines should be loaded.
 -- Currently only one implementation exists, standard-Lua C-side
 -- implementation, LuaJIT-FFI-based one is planned.
-return require 'lgi.corelgilua52'
+local core = require 'lgi.corelgilua52'
+
+-- Helper methods for converting between CamelCase and uscore_delim
+-- names.
+function core.uncamel(name)
+   return core.downcase(name:gsub('([%l%d])([%u])', '%1_%2'))
+end
+
+return core
